/*
 * Copyright (C) 2022 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.exim.epub;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import storybook.model.EntityUtil;
import storybook.model.book.Book;
import storybook.model.hbn.entity.Chapter;
import storybook.model.hbn.entity.Scene;
import storybook.model.hbn.entity.Strand;
import storybook.tools.LOG;
import storybook.tools.xml.Xml;
import storybook.tools.zip.ZipUtil;
import storybook.ui.MainFrame;

/**
 *
 * @author favdb
 */
public class EPUB {

	private static final String TT = "EPUB";

	private String title = "", description = "", author = "", uuid = "", isbn = "";
	private List<String> files;

	public EPUB() {
		files = new ArrayList<>();
	}

	/**
	 * set the title of the book
	 *
	 * @param value
	 */
	public void setTitle(String value) {
		this.title = value;
	}

	/**
	 * get the title of the book
	 *
	 * @return
	 */
	public String getTitle() {
		return title;
	}

	/**
	 * set the author of the book
	 *
	 */
	public void setAuthor(String value) {
		this.author = value;
	}

	/**
	 * get the author of the book
	 *
	 * @return
	 */
	public String getAuthor() {
		return author;
	}

	/**
	 * set the description of the book
	 *
	 * @param value
	 */
	public void setDescription(String value) {
		this.description = value;
	}

	/**
	 * get the description of the book
	 *
	 * @return
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * set the UUID of the book
	 *
	 * @param value
	 */
	public void setUUID(String value) {
		this.uuid = value;
	}

	/**
	 * get the UUID of the book
	 *
	 * @return
	 */
	public String getUUID() {
		return uuid;
	}

	/**
	 * set the ISBN of the book
	 *
	 * @param value
	 */
	public void setISBN(String value) {
		this.isbn = value;
	}

	/**
	 * get the ISBN of the book
	 *
	 * @return
	 */
	public String getISBN() {
		return isbn;
	}

	/**
	 * get the files list
	 *
	 * @return
	 */
	public List<String> getFiles() {
		return files;
	}

	/**
	 * print the resulting content.opf for tracing
	 *
	 */
	public void printOpf() {
		LOG.trace("title=\"" + title + "\"");
		LOG.trace("author=\"" + author + "\"");
		LOG.trace("isbn=\"" + isbn + "\"");
		LOG.trace("uuid=\"" + uuid + "\"");
		LOG.trace("description=\"" + description + "\"");
		LOG.trace("files:");
		for (String f : files) {
			LOG.trace("  - \"" + f + "\"");
		}
	}

	/**
	 * load of the book information from content.opf
	 *
	 * @param zipSrc
	 * @return
	 * @throws IOException
	 */
	public boolean readOPF(String zipSrc) throws IOException {
		LOG.trace(TT + "readOPF(zipSrc=" + zipSrc + ")");
		zipSrc = URLDecoder.decode(zipSrc, "UTF-8");
		ZipFile zipFile = new ZipFile(zipSrc);
		Enumeration<? extends ZipEntry> entries = zipFile.entries();
		while (entries.hasMoreElements()) {
			ZipEntry zipEntry = entries.nextElement();
			if (zipEntry.toString().endsWith(".opf")) {
				InputStream inputStream = zipFile.getInputStream(zipEntry);
				Xml xml = new Xml(inputStream);
				Node metadata = xml.nodeGet("metadata");
				if (metadata == null) {
					return false;
				}
				title = xml.nodeGetValueOfChild(metadata, "dc:title");
				author = xml.nodeGetValueOfChild(metadata, "dc:creator", "opf:role=\"aut\"");
				uuid = xml.nodeGetValueOfChild(metadata, "dc:identifier", "opf:scheme=\"uuid\"");
				isbn = xml.nodeGetValueOfChild(metadata, "dc:identifier", "opf:scheme=\"isbn\"");
				description = xml.nodeGetValueOfChild(metadata, "dc:description");
				Node manifest = xml.nodeGet("manifest");
				if (manifest == null) {
					return false;
				}
				NodeList items = manifest.getChildNodes();
				if (items.getLength() > 0) {
					for (int i = 0; i < items.getLength(); i++) {
						Node item = items.item(i);
						if (Xml.attributeGet(item, "media-type").equals("application/xhtml+xml")) {
							files.add(Xml.attributeGet(item, "href"));
						}
					}
				}
				return true;
			}
		}
		zipFile.close();
		return true;
	}

	/**
	 * read the content of a zipped file as a chapter and a scene
	 *
	 * @param item
	 * @param mainFrame
	 * @return
	 */
	public String readChapters(String item, MainFrame mainFrame) {
		LOG.trace(TT + ".readChapters(item=" + item + ", mainFrame)");
		String rc = "";
		try {
			//create the chapter
			Chapter chapter = new Chapter();
			chapter.setPart(EntityUtil.getFirstPart(mainFrame));
			chapter.setChapterno(EntityUtil.getLastChapterno(mainFrame) + 1);
			chapter.setTitle(chapter.getDefaultName());
			EntityUtil.createEntity(mainFrame, chapter);
			//create the scene
			Document html = Jsoup.parse(new File(item), "utf-8");
			String text = html.body().outerHtml();
			createScene(mainFrame, text);
			return rc;
		} catch (IOException ex) {
			LOG.err(TT + ".readContent error", ex);
			return "";
		}
	}

	private static final String IMG_JPEG = "jpg jpeg";

	/**
	 * create a Scene
	 *
	 * @param mainFrame
	 * @param text
	 */
	private void createScene(MainFrame mainFrame, String text) {
		LOG.trace(TT + ".createScene(mainFrame, text.len=" + text.length() + ")");
		Strand strand = (Strand) EntityUtil.getFirst(mainFrame, Book.TYPE.STRAND);
		Chapter chapter = (Chapter) EntityUtil.getLast(mainFrame, Book.TYPE.CHAPTER);
		int no = EntityUtil.getLastSceneno(mainFrame) + 1;
		Scene scene = new Scene();
		scene.setChapter(chapter);
		scene.setSceneno(no);
		scene.setTitle(String.format("%02d.%02d", chapter.getChapterno(), no));
		scene.setStrand(strand);
		String t = text;
		if (text.contains("<img")) {
			Document doc = Jsoup.parse(text);
			Elements elems = doc.getElementsByTag("img");
			for (int i = 0; i < elems.size(); i++) {
				Element el = elems.get(i);
				String src = el.attr("src");
				if (src.toLowerCase().endsWith(".png")) {
					src = src.replace(".png", ".jpeg").replace(".PNG", ".jpeg");
					el.attr("src", src);
				}
				if (src.toLowerCase().endsWith(".png")) {
					src = src.replace(".bmp", ".jpeg").replace(".BMP", ".jpeg");
					el.attr("src", src);
				}
				File f = new File(src);
				src = "./Images/" + f.getName();
				el.attr("img", src);
			}
			t = doc.body().outerHtml();
		}
		scene.setSummary(t);
		EntityUtil.createEntity(mainFrame, scene);
	}

	/**
	 * copy the JPEG and PNG files to images directory
	 *
	 * @param inZip
	 * @param outDir
	 * @return
	 */
	public boolean copyImages(String inZip, String outDir) {
		LOG.trace(TT + ".copyImages(inZip=\"" + inZip + "\", outDir=\"" + outDir + "\")");
		String fimage = "";
		try {
			String srcDir = URLDecoder.decode(inZip, "UTF-8");
			try (ZipFile zipFile = new ZipFile(srcDir)) {
				Enumeration<? extends ZipEntry> entries = zipFile.entries();
				while (entries.hasMoreElements()) {
					ZipEntry zipEntry = entries.nextElement();
					fimage = zipEntry.toString();
					String simage = fimage;
					if (simage.contains("/")) {
						simage = fimage.substring(fimage.lastIndexOf('/'));
					}
					InputStream zipIs = zipFile.getInputStream(zipEntry);
					if (simage.toLowerCase().endsWith(".jpg")
							|| simage.toLowerCase().endsWith(".jpeg")) {
						ZipUtil.copyTo(zipIs, outDir + File.separator + simage);
					} else if (simage.toLowerCase().endsWith(".png")
							|| simage.toLowerCase().endsWith(".bmp")) {
						simage = simage.replace(".png", ".jpeg").replace(".bmp", ".jpeg");
						PngToJpeg(zipIs, outDir + File.separator + simage);
					}

				}
			}
		} catch (IOException ex) {
			LOG.err("read image=" + fimage + " from=" + inZip + " error", ex);
		}
		return true;
	}

	/**
	 * convert a PNG image to a compressed (65%) JPEG
	 *
	 * @param is
	 * @param outFile
	 * @return
	 */
	public static boolean PngToJpeg(InputStream is, String outFile) {
		try /*( FileOutputStream os = new FileOutputStream(outFile))*/ {
			BufferedImage inImg = ImageIO.read(is);
			ImageWriter writer = ImageIO.getImageWritersByFormatName("jpg").next();
			ImageWriteParam param = writer.getDefaultWriteParam();
			param.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
			param.setCompressionQuality(0.65f);
			writer.setOutput(ImageIO.createImageOutputStream(outFile));
			IIOImage outputImage = new IIOImage(inImg, null, null);
			writer.write(null, outputImage, param);
			writer.dispose();
		} catch (IOException ex) {
			LOG.err("PngToJpeg(...) error", ex);
			return false;
		}
		return true;
	}

}
