/*
Storybook: Open Source software for novelists and authors.
Copyright (C) 2008 - 2012 Martin Mustun

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.model.hbn.entity;

import i18n.I18N;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import org.w3c.dom.Node;
import storybook.model.book.Book;
import storybook.model.hbn.dao.DAOutil;
import static storybook.model.hbn.entity.AbstractEntity.getXmlText;
import static storybook.model.hbn.entity.DB_DATA.DATA.*;
import storybook.tools.TextUtil;

@SuppressWarnings("serial")
public class Chapter extends AbstractEntity {

	private String title = "";
	private Part part;
	private Integer chapterno = 0;
	private Timestamp creationTime;
	private Timestamp objectiveTime;
	private Timestamp doneTime;
	private Integer objectiveChars = 0;
	private Integer words = 0;
	private Integer nbScenes = 0;
	private Integer chars = 0;
	private int intensity;

	public Chapter() {
		super(Book.TYPE.CHAPTER, "111");
	}

	public Chapter(Part part, Integer chapterno, String title,
			String description, String notes,
			Timestamp creationTime, Timestamp objectiveTime,
			Timestamp doneTime, Integer objectiveChars) {
		this();
		this.part = part;
		this.chapterno = chapterno;
		setName(title);
		this.title = title;
		setDescription(description);
		setNotes(notes);
		this.creationTime = creationTime;
		this.objectiveTime = objectiveTime;
		this.doneTime = doneTime;
		this.objectiveChars = objectiveChars;
		setAssistant("");
	}

	public Chapter(Part part, Integer chapterno, String title) {
		this();
		this.part = part;
		this.chapterno = chapterno;
		setName(title);
		this.title = title;
		setDescription("");
		setNotes("");
		this.creationTime = new Timestamp(new Date().getTime());
		this.objectiveChars = 0;
		setAssistant("");
	}

	public Chapter(Long id, Part part, int number, String title) {
		this(part, number, title);
		setId(id);
	}

	public Part getPart() {
		return part;
	}

	public boolean hasPart() {
		return part != null;
	}

	public void setPart(Part part) {
		this.part = part;
	}

	public Integer getChapterno() {
		return (this.chapterno == null ? -1 : this.chapterno);
	}

	public static Integer getNextNumber(List chapters) {
		int n = 0;
		for (Object c : chapters) {
			if (((Chapter) c).chapterno > n) {
				n = ((Chapter) c).chapterno;
			}
		}
		return (n + 1);
	}

	public static Chapter findNumber(List chapters, int n) {
		if (chapters == null || chapters.isEmpty()) {
			return (null);
		}
		for (Object obj : chapters) {
			Chapter c = (Chapter) obj;
			if (c.chapterno == n) {
				return c;
			}
		}
		return (null);
	}

	public void setChapterno(Integer chapterno) {
		this.chapterno = chapterno;
	}

	public String getChapternoStr() {
		String s = String.format("%02d", getChapterno());
		return s;
	}

	public String getDefaultName() {
		String s = String.format("P%02dC%02d", getPart().getNumber(), getChapterno());
		return s;
	}

	public String getTitle() {
		return title;
	}

	public String getTitle(boolean truncate) {
		return TextUtil.truncateString(title, 30);
	}

	public String getTitle(int truncate) {
		return TextUtil.truncateString(title, truncate);
	}

	public void setTitle(String title) {
		this.title = title;
		setName(title);
	}

	public String getNbScenes() {
		return (nbScenes.toString());
	}

	public void setNbScenes(Integer n) {
		nbScenes = n;
	}

	public String getNbWords() {
		return (words.toString());
	}

	public void setNbWords(Integer n) {
		words = n;
	}

	public String getNbChars() {
		return (chars.toString());
	}

	public void setNbChars(Integer n) {
		chars = n;
	}

	public boolean hasCreationTime() {
		return creationTime != null;
	}

	public void setCreationTime(Timestamp ts) {
		creationTime = ts;
	}

	public Timestamp getCreationTime() {
		return creationTime;
	}

	public boolean hasObjectiveTime() {
		return objectiveTime != null;
	}

	public void setObjectiveTime(Timestamp ts) {
		objectiveTime = ts;
	}

	public Timestamp getObjectiveTime() {
		return objectiveTime;
	}

	public boolean isDone() {
		return hasDoneTime();
	}

	public boolean hasDoneTime() {
		return doneTime != null;
	}

	public void setDoneTime(Timestamp ts) {
		doneTime = ts;
	}

	public Timestamp getDoneTime() {
		return doneTime;
	}

	public Integer getObjectiveChars() {
		return (this.objectiveChars == null ? 0 : this.objectiveChars);
	}

	public void setObjectiveChars(Integer objectiveChars) {
		this.objectiveChars = objectiveChars;
	}

	@Override
	public String toString() {
		if (chapterno == null) {
			return I18N.getMsg("scenes.unassigned") + (hasNotes() ? "*" : "");
		}
		return (getChapterno() + ": " + getName() + (hasNotes() ? "*" : ""));
	}

	@Override
	public String toCsv(String quoteStart, String quoteEnd, String separator) {
		StringBuilder b = new StringBuilder();
		b.append(quoteStart).append(getClean(this)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(part)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(chapterno)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getName())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(creationTime)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(objectiveTime)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(doneTime)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(objectiveChars)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getDescription())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getNotes())).append(quoteEnd).append("\n");
		return (b.toString());
	}

	@Override
	public String toHtml() {
		return (toCsv("<td>", "</td>", "\n"));
	}

	@Override
	public String getName() {
		if (super.getName() == null) {
			setName("");
		}
		if (super.getName().isEmpty() && title != null && !title.isEmpty()) {
			setName(title);
		}
		return super.getName();
	}

	public String getFullName() {
		return getChapternoStr() + " " + getName();
	}

	@Override
	public String toDetail(boolean detailed) {
		StringBuilder b = new StringBuilder();
		b.append(toHtmlDetailHeader(detailed));
		if (getPart() != null) {
			b.append(getInfo(DAOutil.PART, getPart().getName()));
		} else {
			b.append(getInfo(DAOutil.PART, "???"));
		}
		b.append(getInfo(DAOutil.NUMBER, getClean(chapterno)));
		if (getObjectiveTime() != null) {
			b.append(getInfo(OBJECTIVE_DATE, getClean(getObjectiveTime())));
			b.append(getInfo(OBJECTIVE_DONE, getClean(getDoneTime())));
			b.append(getInfo(OBJECTIVE_SIZE, getClean(getObjectiveChars())));
		} else if (detailed) {
			b.append(getInfo(OBJECTIVE_DATE, ""));
			b.append(getInfo(OBJECTIVE_DONE, ""));
			b.append(getInfo(OBJECTIVE_SIZE, ""));
		}
		b.append(toHtmlDetailFooter(detailed));
		return (b.toString());
	}

	@Override
	public String toText() {
		return (toCsv("", "", "\t"));
	}

	@Override
	public String toXml() {
		StringBuilder b = new StringBuilder();
		b.append(toXmlHeader());
		b.append(xmlAttribute(DAOutil.PART, getClean(getPart().getId())));
		b.append(xmlAttribute(DAOutil.NUMBER, getClean(chapterno)));
		b.append(xmlAttribute("objdate", getClean(getObjectiveTime())));
		b.append(xmlAttribute("objdone", getClean(getDoneTime())));
		b.append(xmlAttribute("objchars", getClean(getObjectiveChars())));
		b.append(toXmlTab(2)).append(">\n");
		b.append(toXmlEnd());
		return (b.toString());
	}

	public static Chapter fromXml(Node node) {
		Chapter p = new Chapter();
		p.setId(getXmlLong(node, "id"));
		p.setChapterno(getXmlInteger(node, DAOutil.NUMBER));
		p.setTitle(getXmlString(node, DAOutil.NAME).replace("''", "\""));
		p.setCreationTime(new SbDate(getXmlString(node, "creation")).toTimestamp());
		p.setObjectiveTime(new SbDate(getXmlString(node, "objdate")).toTimestamp());
		p.setDoneTime(new SbDate(getXmlString(node, "objdone")).toTimestamp());
		p.setObjectiveChars(getXmlInteger(node, "objchars"));
		p.setDescription(getXmlText(node, "description"));
		p.setNotes(getXmlText(node, "notes"));
		p.setAssistant(getXmlText(node, "assistant"));
		return (p);
	}

	@Override
	@SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
	public boolean equals(Object obj) {
		if (!super.equals(obj)) {
			return false;
		}
		Chapter test = (Chapter) obj;
		boolean ret = true;
		ret = ret && equalsObjectNullValue(part, test.getPart());
		ret = ret && equalsIntegerNullValue(chapterno, test.getChapterno());
		ret = ret && equalsStringNullValue(getName(), test.getName());
		ret = ret && equalsStringNullValue(getDescription(), test.getDescription());
		ret = ret && equalsStringNullValue(getNotes(), test.getNotes());
		return ret;
	}

	@Override
	public int hashCode() {
		return hashPlus(super.hashCode(),
				part,
				chapterno,
				creationTime,
				objectiveChars,
				objectiveTime,
				doneTime);
	}

	@Override
	public int compareTo(AbstractEntity ch) {
		return chapterno.compareTo(((Chapter) ch).chapterno);
	}

	public static Chapter find(List<Chapter> list, String str) {
		for (Chapter elem : list) {
			if (elem.getName().equals(str)) {
				return (elem);
			}
		}
		return (null);
	}

	public static Chapter find(List<Chapter> list, Long id) {
		for (Chapter elem : list) {
			if (elem.id.equals(id)) {
				return (elem);
			}
		}
		return (null);
	}

	public static List<String> getDefColumns() {
		List<String> list = AbstractEntity.getDefColumns(Book.TYPE.CHAPTER);
		list.add("title, 256");
		return (list);
	}

	public static List<String> getTable() {
		List<String> ls = new ArrayList<>();
		String tableName = DAOutil.CHAPTER;
		AbstractEntity.getTable(tableName, ls);
		ls.add(tableName + ",title,String,256");
		ls.add(tableName + ",part_id,Integer,0");
		ls.add(tableName + ",chapterno,Integer,0");
		ls.add(tableName + ",creation_ts,Time,0");
		ls.add(tableName + ",objective_ts,Time,0");
		ls.add(tableName + ",done_ts,Time,0");
		ls.add(tableName + ",objective_ch,Integer,0");
		return (ls);
	}

	/**
	 * get the identification like PP.CC where:
	 * <ul>
	 * <li>PP is the Part ID (may be 00 if there is no Part)</li>
	 * <li>CC is the Chapter ID</li>
	 * </ul>
	 *
	 * @return
	 */
	public String getIdent() {
		if (hasPart()) {
			return String.format("%s.%02d", part.getIdent(), getId());
		}
		return String.format("00.%02d", getId());
	}

}
