/*
Storybook: Open Source software for novelists and authors.
Copyright (C) 2008 - 2012 Martin Mustun

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.model.hbn.entity;

import java.util.ArrayList;
import java.util.List;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import i18n.I18N;
import storybook.model.EntityUtil;
import storybook.model.book.Book;
import storybook.model.hbn.dao.DAOutil;
import storybook.ui.MainFrame;

public class Idea extends AbstractEntity {

	public enum STATUS {
		NOT_STARTED,
		STARTED,
		COMPLETED,
		ABANDONED
	};

	private Integer status = 0;
	private String category = "";
	// TODO changer la catégorie en utilisant la classe Category

	public Idea() {
		super(Book.TYPE.IDEA, "010");
	}

	public Idea(Integer status, String note, String category) {
		this();
		this.status = status;
		setNotes(note);
		this.category = category;
	}

	@Override
	public String getName() {
		if (super.getName().isEmpty()) {
			setName(I18N.getMsg("idea") + " " + getId());
		}
		return super.getName();
	}

	public Integer getStatus() {
		return this.status;
	}

	public void setStatus(Integer status) {
		this.status = status;
	}

	public String getCategory() {
		return this.category;
	}

	public void setCategory(String category) {
		this.category = category;
	}

	@Override
	public String toDetail(boolean detailed) {
		StringBuilder b = new StringBuilder();
		b.append(toHtmlDetailHeader(detailed));
		String str = getStatus().toString() + " (" + Status.getStatusMsg(getStatus()) + ")";
		b.append(getInfo(DAOutil.STATUS, str));
		b.append(getInfo(DAOutil.CATEGORY, getCategory()));
		b.append(toHtmlDetailFooter(detailed));
		return (b.toString());
	}

	@Override
	public String toCsv(String quoteSart, String quoteEnd, String separator) {
		StringBuilder b = new StringBuilder();
		b.append(quoteSart).append(getId().toString()).append(quoteEnd).append(separator);
		b.append(quoteSart).append(getStatus().toString()).append(quoteEnd).append(separator);
		b.append(quoteSart).append(getCategory()).append(quoteEnd).append(separator);
		b.append(quoteSart).append(hasNotes() ? getNotes() : "").append(quoteEnd).append("\n");
		return (b.toString());
	}

	@Override
	public String toHtml() {
		return (toCsv("<td>", "</td>", "\n"));
	}

	@Override
	public String toString() {
		return getName();
	}

	@Override
	public String toText() {
		return (toCsv("", "", "\t"));
	}

	@Override
	public String toXml() {
		StringBuilder b = new StringBuilder();
		b.append(toXmlTab(1)).append("<idea \n");
		b.append(xmlCommon());
		b.append(xmlAttribute("name", "idea " + getId()));
		b.append(xmlAttribute("status", getStatus().toString()));
		b.append(xmlAttribute("category", getCategory()));
		b.append(toXmlTab(2)).append(">\n");
		b.append(toXmlEnd());
		return (b.toString());
	}

	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof Idea) || !super.equals(obj)) {
			return false;
		}
		Idea test = (Idea) obj;
		boolean ret = true;
		ret = ret && equalsIntegerNullValue(status, test.getStatus());
		ret = ret && equalsStringNullValue(category, test.getCategory());
		ret = ret && equalsStringNullValue(getNotes(), test.getNotes());
		return ret;
	}

	@Override
	public int hashCode() {
		int hash = super.hashCode();
		hash = hash * 31 + (status != null ? status.hashCode() : 0);
		hash = hash * 31 + (category != null ? category.hashCode() : 0);
		return hash;
	}

	public static Idea find(List<Idea> list, String str) {
		for (Idea elem : list) {
			if (elem.getName().equals(str)) {
				return (elem);
			}
		}
		return (null);
	}

	public static Idea find(List<Idea> list, Long id) {
		for (Idea elem : list) {
			if (elem.id.equals(id)) {
				return (elem);
			}
		}
		return (null);
	}

	public static List<String> findCategories(MainFrame mainFrame) {
		@SuppressWarnings("unchecked")
		List<Idea> events = EntityUtil.findEntities(mainFrame, Book.TYPE.IDEA);
		List<String> list = new ArrayList<>();
		events.forEach((e) -> {
			if (!list.contains(e.getCategory())) {
				list.add(e.getCategory());
			}
		});
		return (list);
	}

	/**
	 * reading idea from a Xml node
	 *
	 * @param node
	 */
	public Idea(Node node) {
		this();
		if (node.getNodeType() == Node.ELEMENT_NODE) {
			Element eElement = (Element) node;
			//setId(Long.parseLong(eElement.getAttribute("id"))); not usable
			setCreation(eElement.getAttribute("creation"));
			setMaj(eElement.getAttribute("maj"));
			setName(eElement.getAttribute("name"));
			setStatus(Integer.parseInt(eElement.getAttribute("status")));
			setCategory(eElement.getAttribute("category"));
			setNotes(eElement.getElementsByTagName("notes").item(0).getTextContent());
		}
	}

	public void toXml(Document doc, Node node) {
		if (doc == null) {
			return;
		}
		if (node == null) {
			node = doc.createElement("idea");
		}
		if (node.getNodeType() == Node.ELEMENT_NODE) {
			Element element = (Element) node;
			element.setAttribute("creation", getCreation());
			element.setAttribute("maj", getMaj());
			element.setAttribute("name", getName());
			element.setAttribute("status", getStatus().toString());
			element.setAttribute("category", getCategory());
			NodeList inode = element.getElementsByTagName("notes");
			if (inode.getLength() < 1) {
				Element item = doc.createElement("idea");
				node.appendChild(item);
				inode = element.getElementsByTagName("notes");
			}
			inode.item(0).setTextContent(getNotes());
		}
	}

	public static List<String> getDefColumns() {
		List<String> list = AbstractEntity.getDefColumns(Book.TYPE.IDEA);
		list.add("category, 256");
		return (list);
	}

	public static List<String> getTable() {
		List<String> ls = new ArrayList<>();
		String tableName = "ideas";
		AbstractEntity.getTable(tableName, ls);
		ls.add(tableName + ",status,Integer,0");
		ls.add(tableName + ",category,String,256");
		return (ls);
	}

}
