package storybook.tools.file;

import java.io.File;
import java.io.IOException;
import java.nio.file.FileSystems;
import java.nio.file.Path;
import java.nio.file.StandardWatchEventKinds;
import java.nio.file.WatchEvent;
import java.nio.file.WatchKey;
import java.nio.file.WatchService;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * The Class FileWatcher.
 *
 * @author Jean Rebillat
 */
public class FileWatcher extends Thread {

	/**
	 * The file.
	 */
	private final File file;

	/**
	 * The stop.
	 */
	private AtomicBoolean stop = new AtomicBoolean(false);

	/**
	 * The listeners.
	 */
	List<FileWatcherListener> listeners = new LinkedList<>();

	/**
	 * Instantiates a new file watcher.
	 *
	 * @param file the file
	 * @param listener the listener
	 */
	public FileWatcher(File file, FileWatcherListener listener) {
		this.file = file;
		listeners.add(listener);
	}

	/**
	 * Checks if is stopped.
	 *
	 * @return true, if is stopped
	 */
	public boolean isStopped() {
		return stop.get();
	}

	/**
	 * Stop thread.
	 */
	public void stopThread() {
		stop.set(true);
	}

	/**
	 * Adds the listener.
	 *
	 * @param listener the listener
	 */
	void addListener(FileWatcherListener listener) {
		if (!listeners.contains(listener)) {
			listeners.add(listener);
		}
	}

	/**
	 * Removes the listener.
	 *
	 * @param listener the listener
	 */
	void removeListener(FileWatcherListener listener) {
		if (listeners.contains(listener)) {
			listeners.remove(listener);
		}
	}

	/**
	 * Run.
	 */
	@Override
	@SuppressWarnings("CallToThreadYield")
	public void run() {
		try (WatchService watcher = FileSystems.getDefault().newWatchService()) {
			Path path = file.toPath().getParent();
			path.register(watcher, StandardWatchEventKinds.ENTRY_MODIFY);
			while (!isStopped()) {
				WatchKey key;
				key = watcher.take();
				if (key == null) {
					Thread.yield();
					continue;
				}
				for (WatchEvent<?> event : key.pollEvents()) {
					WatchEvent.Kind<?> kind = event.kind();
					@SuppressWarnings("unchecked")
					WatchEvent<Path> ev = (WatchEvent<Path>) event;
					Path filename = ev.context();
					if (kind == StandardWatchEventKinds.OVERFLOW) {
						Thread.yield();
					} else if (kind == java.nio.file.StandardWatchEventKinds.ENTRY_MODIFY
							&& filename.toString().equals(file.getName())) {
						for (FileWatcherListener listener : listeners) {
							listener.onEvent(FileWatcherListener.EventType.MODIFY, file);
						}
					}
				}
				boolean valid = key.reset();
				if (!valid) {
					break;
				}
			}
			Thread.yield();
		} catch (RuntimeException | IOException | InterruptedException e) {
			Thread.currentThread().interrupt();
		}
	}

	/**
	 * The listener interface for receiving watcher events. The class that is interested in processing a watcher event
	 * implements this interface, and the object created with that class is registered with a component using the
	 * component's <code>addWatcherListener<code> method. When
	 * the watcher event occurs, that object's appropriate
	 * method is invoked.
	 *
	 * @see WatcherEvent
	 */
	@FunctionalInterface
	public interface FileWatcherListener {

		/**
		 * The Enum EventType.
		 */
		public enum EventType {
			CREATE,//The create
			DELETE,//The delete
			MODIFY//The modify
		}

		/**
		 * On event.
		 *
		 * @param type the type
		 * @param file the file
		 */
		public void onEvent(EventType type, File file);
	}
}
