/*
 * Copyright (C) 2022 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.tools.xml;

import i18n.I18N;
import java.io.IOException;
import java.io.InputStream;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import storybook.tools.LOG;
import storybook.ui.dialog.ExceptionDlg;

/**
 *
 * @author favdb
 */
public class Xml {

	public static final String HEADER = "<?xml version=\"1.0\""
			+ " encoding=\"UTF-8\""
			+ " standalone=\"no\"?>\n";
	private DocumentBuilder documentBuilder;
	private final InputStream stream;
	private final Document document;
	private Element rootNode;

	public Xml(InputStream inputStream) {
		stream = inputStream;
		documentBuilder = null;
		try {
			documentBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		} catch (ParserConfigurationException ex) {
			ExceptionDlg.show("", new Exception(I18N.getMsg("import.dlg.open.docbuilder")));
			LOG.err("DocumentBuilder error:\n", ex);
		}
		document = readDom();
		if (document == null) {
			rootNode = null;
		} else {
			rootNode = document.getDocumentElement();
		}
	}

	public Document readDom() {
		//LOG.trace("Importer.readDom()");
		Document rc = null;
		try {
			rc = documentBuilder.parse(stream);
		} catch (SAXException ex) {
			ExceptionDlg.show("", new Exception(I18N.getMsg("import.dlg.open.parsing")));
			LOG.err("Parsing XML error", ex);
		} catch (IOException ex) {
			ExceptionDlg.show("", new Exception(I18N.getMsg("import.dlg.open.io")));
			LOG.err("I/O error", ex);
		}
		return rc;
	}

	public Node nodeGet(String key) {
		return rootNode.getElementsByTagName(key).item(0);
	}

	public String nodeGetValueOfChild(Node node, String child) {
		NodeList childs = ((Element) node).getElementsByTagName(child);
		if (childs.getLength() > 0) {
			return childs.item(0).getTextContent();
		}
		return "none";
	}

	public String nodeGetValueOfChild(Node node, String child, String qualifier) {
		NodeList childs = ((Element) node).getElementsByTagName(child);
		if (childs.getLength() > 0) {
			for (int i = 0; i < childs.getLength(); i++) {
				if (XmlUtil.NodeToString(childs.item(i)).toLowerCase().contains(qualifier.toLowerCase())) {
					return childs.item(i).getTextContent();
				}
			}
		}
		return "";
	}

	/**
	 * get attribute value of a Node
	 *
	 * @param node: the Node
	 * @param key: the attribute key name
	 * @return
	 */
	public static String attributeGet(Node node, String key) {
		NamedNodeMap attrs = node.getAttributes();
		if (attrs != null) {
			for (int i = 0; i < attrs.getLength(); i++) {
				Node attr = attrs.item(i);
				if (attr.getNodeName().equalsIgnoreCase(key)) {
					return attr.getNodeValue().trim();
				}
			}
		}
		return "";
	}

	/**
	 * get attribute value of a String Node
	 *
	 * @param node: the Node String
	 * @param key: the attribute key name
	 * @return
	 */
	public static String attributeGet(String node, String key) {
		int hstart = node.toLowerCase().indexOf(key.toLowerCase()) + key.toLowerCase().length();
		String sx = node.substring(hstart);
		sx = sx.substring(0, sx.indexOf("\""));
		return sx;
	}

}
